/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2008-2010 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "BasicReactingMultiphaseParcel.H"

// Kinematic
#include "makeReactingParcelDispersionModels.H"
#include "makeReactingParcelDragModels.H"
#include "makeReactingMultiphaseParcelInjectionModels.H" // MP variant
#include "makeReactingParcelCollisionModels.H"
#include "makeReactingParcelPatchInteractionModels.H"
#include "makeReactingParcelPostProcessingModels.H"

// Thermodynamic
#include "makeReactingParcelHeatTransferModels.H"

// Reacting
#include "makeReactingMultiphaseParcelCompositionModels.H" // MP variant
#include "makeReactingParcelPhaseChangeModels.H"

// Reacting multiphase
#include "makeReactingMultiphaseParcelDevolatilisationModels.H"
#include "makeReactingMultiphaseParcelSurfaceFilmModels.H"
#include "makeReactingMultiphaseParcelSurfaceReactionModels.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
    // Kinematic sub-models
    makeReactingDispersionModels(BasicReactingMultiphaseParcel);
    makeReactingDragModels(BasicReactingMultiphaseParcel);
    makeReactingMultiphaseInjectionModels(BasicReactingMultiphaseParcel);
    makeReactingCollisionModels(BasicReactingMultiphaseParcel);
    makeReactingPatchInteractionModels(BasicReactingMultiphaseParcel);
    makeReactingPostProcessingModels(BasicReactingMultiphaseParcel);

    // Thermo sub-models
    makeReactingHeatTransferModels(BasicReactingMultiphaseParcel);

    // Reacting sub-models
    makeReactingMultiphaseCompositionModels(BasicReactingMultiphaseParcel);
    makeReactingPhaseChangeModels(BasicReactingMultiphaseParcel);

    // Reacting multiphase sub-models
    makeReactingMultiphaseDevolatilisationModels
    (
        BasicReactingMultiphaseParcel
    );
    makeReactingMultiphaseSurfaceFilmModels
    (
        BasicReactingMultiphaseParcel
    );
    makeReactingMultiphaseSurfaceReactionModels
    (
        BasicReactingMultiphaseParcel
    );
};


// ************************************************************************* //
