/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2010 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*----------------------------------------------------------------------------*/

#include "thermalPorousZone.H"
#include "fvMesh.H"
#include "fvMatrices.H"
#include "basicThermo.H"

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::thermalPorousZone::thermalPorousZone
(
    const keyType& key,
    const fvMesh& mesh,
    const dictionary& dict
)
:
    porousZone(key, mesh, dict),
    T_("T", dimTemperature, -GREAT)
{
    if (const dictionary* dictPtr = dict.subDictPtr("thermalModel"))
    {
        const word thermalModel(dictPtr->lookup("type"));

        if (thermalModel == "fixedTemperature")
        {
            dictPtr->lookup("T") >> T_;
        }
        else
        {
            FatalIOErrorIn
            (
                "thermalPorousZone::thermalPorousZone"
                "(const keyType&, const fvMesh&, const dictionary&)",
                *dictPtr
            )   << "thermalModel " << thermalModel << " is not supported" << nl
                << "    Supported thermalModels are: fixedTemperature"
                << exit(FatalIOError);
        }
    }
}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void Foam::thermalPorousZone::addEnthalpySource
(
    const basicThermo& thermo,
    const volScalarField& rho,
    fvScalarMatrix& hEqn
) const
{
    const labelList& zones = this->zoneIds();
    if (zones.empty() || T_.value() < 0.0)
    {
        return;
    }

    const scalarField& V = mesh().V();
    scalarField& hDiag = hEqn.diag();
    scalarField& hSource = hEqn.source();

    // TODO: generalize for non-fixedTemperature methods
    const scalar rate = 1e6;

    forAll(zones, zoneI)
    {
        const labelList& cells = mesh().cellZones()[zones[zoneI]];

        forAll(cells, i)
        {
            hDiag[cells[i]] += rate*V[cells[i]]*rho[cells[i]];
            hSource[cells[i]] += rate*V[cells[i]]*rho[cells[i]]*T_.value();
        }
    }
}


// ************************************************************************* //
